<?php

declare(strict_types=1);


class CapabilityChannelController
{
    const capabilityPrefix = 'ChannelController';
    const DATE_TIME_FORMAT = 'o-m-d\TH:i:s\Z';

    use HelperCapabilityDiscovery;
    use HelperChannelDevice;

    private static function computePropertiesForValue($value)
    {
        return [
            [
                'namespace'                 => 'Alexa.ChannelController',
                'name'                      => 'channel',
                'value'                     => $value,
                'timeOfSample'              => gmdate(self::DATE_TIME_FORMAT),
                'uncertaintyInMilliseconds' => 0
            ]
        ];
    }

    public static function computeProperties($configuration)
    {
        if (IPS_VariableExists($configuration[self::capabilityPrefix . 'ID'])) {
            return self::computePropertiesForValue(self::getChannelValue($configuration[self::capabilityPrefix . 'ID']));
        } else {
            return [];
        }
    }

    public static function getColumns()
    {
        return [
            [
                'label' => 'Channel Variable',
                'name'  => self::capabilityPrefix . 'ID',
                'width' => '150px',
                'add'   => 0,
                'edit'  => [
                    'type' => 'SelectVariable'
                ]
            ]
        ];
    }

    public static function getStatus($configuration)
    {
        return self::getChannelCompatibility($configuration[self::capabilityPrefix . 'ID']);
    }

    public static function getStatusPrefix()
    {
        return 'Channel: ';
    }

    public static function doDirective($configuration, $directive, $payload, $emulateStatus)
    {
        $switchChannel = function ($configuration, $value, $emulateStatus)
        {
        	if ($check=self::setChannel($configuration[self::capabilityPrefix . 'ID'], $value)) {
                $properties = [];
                if ($emulateStatus) {
                    $properties = self::computePropertiesForValue($value);
                } else {
                    $i = 0;
                    while (($value != self::getChannelNumber($configuration[self::capabilityPrefix . 'ID'])) && $i < 10) {
                        $i++;
                        usleep(100000);
                    }
                    $properties = self::computeProperties($configuration);
                }
                return [
                    'properties'     => $properties,
                    'payload'        => new stdClass(),
                    'eventName'      => 'Response',
                    'eventNamespace' => 'Alexa'
                ];
            } else {
            	if(is_null($check)){
            		$payload =  [
                        'type' 		=> 'NO_SUCH_ENDPOINT'
                    ];
            		
            	} else {
            		$payload =  [
                        'type' 		=> 'INVALID_VALUE',
                    	'message'	=> "Channel $value not found in profile associations"
            		];
            	}
                return [
                    'payload'        => $payload,
                    'eventName'      => 'ErrorResponse',
                    'eventNamespace' => 'Alexa'
                ];
            }
        };
        
        $skipChannels = function ($configuration, $step, $emulateStatus)
        {
        	$newChannel=0;
            if (self::skipChannels($configuration[self::capabilityPrefix . 'ID'], $step, $newChannel)) {
                $properties = [];
                if ($emulateStatus) {
                    $properties = self::computePropertiesForValue($newChannel);
                } else {
                  	$i = 0;
                    while (($newChannel != self::getChannelNumber($configuration[self::capabilityPrefix . 'ID'])) && $i < 10) {
                        $i++;
                        usleep(100000);
                    }
                    $properties = self::computeProperties($configuration);
                }
                return [
                    'properties'     => $properties,
                    'payload'        => new stdClass(),
                    'eventName'      => 'Response',
                    'eventNamespace' => 'Alexa'
                ];
            } else {
                return [
                    'payload'        => [
                        'type' => 'NO_SUCH_ENDPOINT'
                    ],
                    'eventName'      => 'ErrorResponse',
                    'eventNamespace' => 'Alexa'
                ];
            }
        };
        
        
        switch ($directive) {
            case 'ReportState':
                return [
                    'properties'     => self::computeProperties($configuration),
                    'payload'        => new stdClass(),
                    'eventName'      => 'StateReport',
                    'eventNamespace' => 'Alexa'
                ];
                break;

            case 'ChangeChannel':
           	if( empty($payload['channel']['number'])){
            		$newValue = $payload['channelMetadata']['name'];
            	} else {
            		$newValue = $payload['channel']['number'];
            	}
            	return $switchChannel($configuration, $newValue, $emulateStatus);
            case 'SkipChannels':;	
            	$step = $payload['channelCount'];
            	return $skipChannels($configuration, $step, $emulateStatus);
            default:
                throw new Exception('Command is not supported by this trait!');
        }
    }

    public static function getObjectIDs($configuration)
    {
        return [
            $configuration[self::capabilityPrefix . 'ID']
        ];
    }

    public static function supportedDirectives()
    {
        return [
            'ReportState',
            'ChangeChannel',
            'SkipChannels'
        ];
    }

    public static function supportedCapabilities()
    {
        return [
            'Alexa.ChannelController'
        ];
    }

    public static function supportedProperties($realCapability, $configuration)
    {
        return [
            'channel'
        ];
    }
}
