<?php

declare(strict_types=1);

class CapabilityEqualizerController
{
    const capabilityPrefix = 'EqualizerController';
    const DATE_TIME_FORMAT = 'o-m-d\TH:i:s\Z';
    

    use HelperDimDevice;
    
    private static function computePropertiesForBands($bands){
    	$values = [];
    	if(is_array($bands)){
    		foreach($bands as $band){
    			$values[]=['name'=>$band['name'], 'value'=>intval($band['level']) ];
		  	}
    	}
    	return [ 
    		[
                'namespace'                 => 'Alexa.EqualizerController',
                'name'                      => 'bands',
                'value'                     => $values,
        		'timeOfSample'              => gmdate(self::DATE_TIME_FORMAT),
                'uncertaintyInMilliseconds' => 0
            ]
    	];		   	
    }
    
    public static function computeProperties($configuration)
    {
    	$bands = [];
    	foreach(['BASS','MIDRANGE','TREBLE'] as $bandname){
	    	if (IPS_VariableExists($configuration[self::capabilityPrefix . $bandname. 'ID'])) {
	    		$bands[]=[
	    			'name'=>$bandname,
	    			'level'=>self::getDimValue($configuration[self::capabilityPrefix . $bandname.'ID'])
	    		];
    		}
    	}
    	return self::computePropertiesForBands($bands);
    }

    public static function getColumns()
    {
        return [
            [
                'label' => 'Bass Variable',
                'name'  => self::capabilityPrefix . 'BASSID',
                'width' => '250px',
                'add'   => 0,
                'edit'  => [
                    'type' => 'SelectVariable'
                ]
            ],
        	[
                'label' => 'Midrange Variable',
                'name'  => self::capabilityPrefix . 'MIDRANGEID',
                'width' => '250px',
                'add'   => 0,
                'edit'  => [
                    'type' => 'SelectVariable'
                ]
            ],
        	[
                'label' => 'Treble Variable',
                'name'  => self::capabilityPrefix . 'TREBLEID',
                'width' => '250px',
                'add'   => 0,
                'edit'  => [
                    'type' => 'SelectVariable'
                ]
            ]
        ];
    }

    public static function getStatus($configuration)
    {
        $bassOk 	= self::getDimCompatibility($configuration[self::capabilityPrefix . 'BASSID']);
        $midOk	 	= self::getDimCompatibility($configuration[self::capabilityPrefix . 'MIDRANGEID']);
        $trebleOk	= self::getDimCompatibility($configuration[self::capabilityPrefix . 'TREBLEID']);
        if($bassOk=='Missing' && $midOk=='Missing' && $trebleOk=='Missing'){
        	return 'Bass '.$bassOk;
        }
       	if($bassOk=='Missing') $bassOk='OK';
		if($midOk=='Missing') $midOk='OK';
       	if($trebleOk=='Missing')$trebleOk='OK';
       	if($bassOk!='OK'){
       		$ok='Bass '.$bassOk;
       	}
       	elseif($midOk!='OK'){
       		$ok= 'Midrange '.$midOk;
       	}
       	elseif($trebleOk!='OK'){
       		$ok='Treble '.$trebleOk;
       	}
       	else $ok='OK';
       	
       	return  $ok;
    }

    public static function getStatusPrefix()
    {
        return 'Equalizer: ';
    }

    public static function doDirective($configuration, $directive, $payload, $emulateStatus)
    {
        $setBands = function ($configuration, $bands, $emulateStatus)
        {
        	$lastBand = null;
        	foreach($bands as &$band){
        		if(self::dimDevice($configuration[self::capabilityPrefix . $band['name']. 'ID'], $band['level'])){
        			$lastBand = $band;
        		}
        	}
        	if($lastBand){
                $properties = [];
                if ($emulateStatus) {
                    $properties = self::computePropertiesForBands($bands);
                } else {
                    $i = 0;
                    while (($lastBand['level'] != self::getDimValue($configuration[self::capabilityPrefix . $lastBand['name']. 'ID'])) && $i < 10) {
                        $i++;
                        usleep(100000);
                    }
                    $properties = self::computeProperties($configuration);
                }
                return [
                    'properties'     => $properties,
                    'payload'        => new stdClass(),
                    'eventName'      => 'Response',
                    'eventNamespace' => 'Alexa'
                ];
            } else {
                return [
                    'payload'        => [
                        'type' => 'NO_SUCH_ENDPOINT'
                    ],
                    'eventName'      => 'ErrorResponse',
                    'eventNamespace' => 'Alexa'
                ];
            }
        };
       
        $adjustBands = function ($configuration, $bands, $emulateStatus) use ($setBands)
        {
        	$newBands = [];
        	foreach($bands as $band){
        		if(IPS_VariableExists($configuration[self::capabilityPrefix . $band['name']. 'ID'])){
					$value = self::getDimValue($configuration[self::capabilityPrefix . $band['name']. 'ID']);
        			if(empty($band['levelDelta']))$band['levelDelta']=1;
 					$step=$band['levelDirection']=='UP' ? $band['levelDelta'] : -$band['levelDelta'];
        			$newBands[]=['name'=>$band['name'],'level'=> $value + $step];
        		}
        		
        	}
        	return $setBands($configuration,$newBands,$emulateStatus);
        };
        
        $resetBands = function ($configuration, $bands, $emulateStatus) use ($setBands)
        {
        	$newBands = [];
        	foreach($bands as $band){
        		if(IPS_VariableExists($configuration[self::capabilityPrefix . $band['name']. 'ID'])){
					$newBands[]=['name'=>$band['name'],'level'=> 0];
        		}
        		
        	}
        	return $setBands($configuration,$newBands,$emulateStatus);
        };
        
        
        switch ($directive) {
            case 'ReportState':
                return [
                    'properties'     => self::computeProperties($configuration),
                    'payload'        => new stdClass(),
                    'eventName'      => 'StateReport',
                    'eventNamespace' => 'Alexa'
                ];
                break;
            case 'AdjustBands':
                return $adjustBands($configuration,$payload['bands'] , $emulateStatus);
            case 'ResetBands': 
                return $resetBands($configuration, $payload['bands'], $emulateStatus);
            case 'SetBands':
                return $setBands($configuration, $payload['bands'], $emulateStatus);
            default:
                throw new Exception('Command is not supported by this trait!');
        }
    }

    public static function getObjectIDs($configuration)
    {
    	$ids = [];
    	foreach(['BASSID','MIDRANGEID','TREBLEID'] as $id){
    		$varid = $configuration[self::capabilityPrefix .$id];
    		if($varid)$ids[]=$varid;
    	}
    	return $ids;
    }

    public static function supportedDirectives()
    {
        return [
            'ReportState',
            'SetBands',
            'AdjustBands',
        	'ResetBands'	
        ];
    }

    public static function supportedCapabilities()
    {
        return [
            'Alexa.EqualizerController'
        ];
    }

    public static function supportedProperties($realCapability, $configuration)
    {
        return [
            'bands'
        ];
    }
    
    private static function calculateRange($configuration){
    	$min=$max=0;
    	$profiles=[];
    	foreach(['BASSID','MIDRANGEID','TREBLEID'] as $varIdent){
    		$variableID=$configuration[self::capabilityPrefix . $varIdent];
    		if (!IPS_VariableExists($variableID)) {
    			continue;
    		}
    		$targetVariable = IPS_GetVariable($variableID);
	        $profileName = ($targetVariable['VariableCustomProfile'] != '') ? $targetVariable['VariableCustomProfile'] : $targetVariable['VariableProfile'];
	       	$profiles[$profileName]=true;
    	}
    	foreach(array_keys($profiles) as $profileName){
	       	$profile = IPS_GetVariableProfile($profileName);
			if ($profile['MaxValue']>$max) $max = $profile['MaxValue'];
	        if ($profile['MinValue']<$min) $min = $profile['MinValue'];
    	}
	    return ['minimum'=>$min, 'maximum'=>$max];
    }
    private static function computeConfiguration($realCapability,$configuration){
		$config = [];    	
    	foreach(['BASS','MIDRANGE','TREBLE'] as $bandname){
	    	if (IPS_VariableExists($configuration[self::capabilityPrefix . $bandname. 'ID'])) {
	    		$config['configurations']['bands']['supported'][]=['name'=>$bandname];
   			}
    	}
    	$config['configurations']['bands']['range'] = self::calculateRange($configuration);
   		return $config;
    }
    
   	public static function getCapabilityInformation($configuration)
    {
        $capabilitiesInfo = [];
        $capabilities = self::supportedCapabilities();
        foreach ($capabilities as $realCapability) {
            $supportedProperties = [];
            $supportedPropertiesNames = self::supportedProperties($realCapability, $configuration);
            if ($supportedPropertiesNames != null) {
                foreach ($supportedPropertiesNames as $property) {
                    $supportedProperties[] = [
                        'name' => $property
                    ];
                }
                $info = [
                    'type'       => 'AlexaInterface',
                    'interface'  => $realCapability,
                    'version'    => '3',
                    'properties' => [
                        'supported'           => $supportedProperties,
                        'proactivelyReported' => false,
                        'retrievable'         => true
                    ]

                ];
                $config = self::computeConfiguration($realCapability, $configuration);
                if(is_array($config)){
             		$info = array_merge($info,$config);
                }
                $capabilitiesInfo[] = $info;
            }
        }
        return $capabilitiesInfo;
    }
    
}
